/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * <BR><BR>
 *
 * Author: John Thompson, Greenstone Digital Library, University of Waikato
 *
 * <BR><BR>
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * <BR><BR>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * <BR><BR>
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * <BR><BR>
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */
package org.greenstone.gatherer.gui;

import java.awt.*;
import java.awt.event.*;
import java.net.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import org.greenstone.gatherer.Dictionary;
import org.greenstone.gatherer.Gatherer;
import org.greenstone.gatherer.WGet;
import org.greenstone.gatherer.util.Utility;

/**
 * @author John Thompson, Greenstone Digital Library, University of Waikato
 * @version 2.1
 */
public class MirrorPane
    extends JPanel {
    private JButton download_button = null;
    private JCheckBox automatic_checkbox = null;
    private JCheckBox higher_checkbox = null;
    private JCheckBox overwrite_checkbox = null;
    private JCheckBox remove_failed_checkbox = null;
    private JCheckBox same_host_checkbox = null;
    private JRadioButton public_radiobutton = null;
    private JRadioButton private_radiobutton = null;
    private JScrollPane list_scroll = null;
    private JSpinner depth_spinner = null;
    private JTextField url_textfield = null;
    private WGet getter = null;

    public MirrorPane() {
	super();
	// Create a new wget thread
        getter = new WGet();
        getter.start();
	// And retrieve its list of pending jobs
	list_scroll = getter.getJobList();

	// Create
	JPanel edit_pane = new JPanel();
	JPanel details_pane = new JPanel();

	JPanel url_pane = new JPanel();
	JLabel url_label = new JLabel();
	url_label.setPreferredSize(Utility.LABEL_SIZE);
	Dictionary.registerText(url_label, "Mirroring.Source_URL");
	url_textfield = new JTextField();
	Dictionary.registerTooltip(url_textfield, "Mirroring.Source_URL_Tooltip");

	JPanel depth_pane = new JPanel();
	JLabel depth_label = new JLabel();
	Dictionary.registerText(depth_label, "Mirroring.Download_Depth");
	depth_spinner = new JSpinner();
	Dictionary.registerTooltip(depth_spinner, "Mirroring.Download_Depth_Tooltip");

	JPanel destination_pane = new JPanel();
	JLabel destination_label = new JLabel();
	Dictionary.registerText(destination_label, "Mirroring.Destination_Folder");
	ButtonGroup group = new ButtonGroup();
	public_radiobutton = new JRadioButton();
	public_radiobutton.setBackground(Gatherer.config.getColor("coloring.collection_tree_background", false));
	public_radiobutton.setOpaque(false);
	Dictionary.registerText(public_radiobutton, "Tree.Public");
	private_radiobutton = new JRadioButton();
	private_radiobutton.setBackground(Gatherer.config.getColor("coloring.collection_tree_background", false));
	private_radiobutton.setOpaque(false);
	Dictionary.registerText(private_radiobutton, "Tree.Private");
	group.add(public_radiobutton);
	group.add(private_radiobutton);
	public_radiobutton.setSelected(true);

	JPanel first_pane = new JPanel();
	automatic_checkbox = new JCheckBox();
	Dictionary.registerText(automatic_checkbox, "Mirroring.Download_Hidden");
	overwrite_checkbox = new JCheckBox();
	Dictionary.registerText(overwrite_checkbox, "Mirroring.Overwrite_Existing");

	JPanel second_pane = new JPanel();
	higher_checkbox = new JCheckBox();
	Dictionary.registerText(higher_checkbox, "Mirroring.Higher_Directories");
	same_host_checkbox = new JCheckBox();
	Dictionary.registerText(same_host_checkbox, "Mirroring.Same_Host");

	remove_failed_checkbox = new JCheckBox();
	Dictionary.registerText(remove_failed_checkbox, "Mirroring.Remove_Failed");

	JLabel further_label = new JLabel();
	Dictionary.registerText(further_label, "Mirroring.Further_Options");

	JPanel button_pane = new JPanel();
	download_button = new JButton();
	Dictionary.registerBoth(download_button, "Mirroring.Download", "Mirroring.Download_Tooltip");

	// Connect
	download_button.addActionListener(new DownloadButtonListener());

	// Layout
	url_pane.setLayout(new BorderLayout());
	url_pane.add(url_label, BorderLayout.WEST);
	url_pane.add(url_textfield, BorderLayout.CENTER);

	depth_pane.setLayout(new GridLayout(1,3));
	depth_pane.add(depth_label);
	depth_pane.add(new JPanel());
	depth_pane.add(depth_spinner);

	destination_pane.setLayout(new GridLayout(1,3));
	destination_pane.add(destination_label);
	destination_pane.add(public_radiobutton);
	destination_pane.add(private_radiobutton);

	first_pane.setLayout(new GridLayout(1,2));
	first_pane.add(automatic_checkbox);
	first_pane.add(overwrite_checkbox);

	second_pane.setLayout(new GridLayout(1,2));
	second_pane.add(higher_checkbox);
	second_pane.add(same_host_checkbox);

	details_pane.setLayout(new GridLayout(7,1,0,5));
	details_pane.add(url_pane);
	details_pane.add(depth_pane);
	details_pane.add(destination_pane);
	details_pane.add(first_pane);
	details_pane.add(second_pane);
	details_pane.add(remove_failed_checkbox);
	details_pane.add(further_label);

	button_pane.setLayout(new GridLayout(1,3));
	button_pane.add(new JPanel());
	button_pane.add(new JPanel());
	button_pane.add(download_button);

	edit_pane.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createEmptyBorder(2,0,0,0), BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder(Dictionary.get("Mirroring.Download_Controls")), BorderFactory.createEmptyBorder(2,2,2,2))));
	edit_pane.setLayout(new BorderLayout());
	edit_pane.add(details_pane, BorderLayout.CENTER);
	edit_pane.add(button_pane, BorderLayout.SOUTH);

	setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
	setLayout(new BorderLayout());
	add(edit_pane, BorderLayout.NORTH);
	add(list_scroll, BorderLayout.CENTER);
    }

    public void collectionChanged(boolean ready) {
	if(private_radiobutton.isSelected() && !ready) {
	    public_radiobutton.setSelected(true);
	}
	private_radiobutton.setEnabled(ready);
    }

    public void setURL(String url) {
	url_textfield.setText(url);
    }

    private class DownloadButtonListener
	implements ActionListener {

	public void actionPerformed(ActionEvent event) {
	    // Retrieve the current url and confirm it is valid
	    String url_str = url_textfield.getText();
	    URL url = null;
	    try {
		url = new URL(url_str);
	    }
	    catch(MalformedURLException error) {
		JOptionPane.showMessageDialog(Gatherer.g_man, Dictionary.get("Mirroring.Invalid_URL"), Dictionary.get("Mirroring.Invalid_URL__Title"), JOptionPane.ERROR_MESSAGE);
	    }
	    if(url != null) {
		// Otherwise construct a new download job
		getter.newJob(Gatherer.c_man.getWorkspace(), overwrite_checkbox.isSelected(), !higher_checkbox.isSelected(), !same_host_checkbox.isSelected(), automatic_checkbox.isSelected(), url, ((Integer)depth_spinner.getValue()).intValue(), (public_radiobutton.isSelected() ? Utility.CACHE_DIR : Gatherer.c_man.getCollectionCache()));
	    }
	}
    }
}
